/*  generate Povray file "povray.c"     -*- C -*-   */
/*  for POVRAY */

/*
#S#
#
# This Program is part of SV (Shelx-output Viewer)
# Copyright (C) 1991-1998 Takashi Nemoto
#
#    This program is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details. (File "COPYING")
#
#    Note!: General Public Licence is copyrighted by the Free Software 
#           Foundation,  but this code is copyrighted by me.
#
#    Send bugs and comments to tnemoto@eels.kuicr.kyoto-u.ac.jp
#
#E#
*/

#define __povray_c__
#define __cmd_c_proc__
#define __subs_c_proc__
#define __string_proc__

#include "def.h"
#include <ctype.h>

SV_LocalFunction int povray_out_header(FILE *fp)
{
  fprintf(fp,
"#include \"colors.inc\"\n"
"#include \"functions.inc\"\n"
"#include \"shapes.inc\"\n"

"#declare SCALE = 1.0;\n"
"#declare TRANSLATE = <0,0,0>;\n"
"#declare DISTANCE = 30;\n"
"#declare STEREO = 0;\n"
"#declare STEREOSPLIT = 5;\n"
"#declare PERSPECTIVE = 0;\n"
"#declare ASPECTRATIO = image_width/image_height;\n"

"global_settings {\n"
"//        assumed_gamma 1\n"
"}\n"

"#declare CENTER = <%.3f,%.3f,%.3f>;\n"
"#declare CamLoc = < 0, 0, -DISTANCE>;\n"

"light_source {CamLoc color rgb <1,1,1> adaptive 1}\n"
"light_source {< 25, 25, 50> color rgb <1,1,1>  adaptive 1 }\n"
"light_source {< 0, 0, 100> color rgb <1,1,1>  adaptive 1 shadowless}\n"
"// light_source {< 0, 0, 100> color rgb <1,1,1>  adaptive 1 shadowless}\n"
"// light_source {< 0, 100, 0> color rgb <1,1,1>  adaptive 1 shadowless}\n"
"// light_source {< 0, -100, 0> color rgb <1,1,1>  adaptive 1 shadowless}\n"
"// light_source {<100, 0, 0> color rgb <1,1,1>  adaptive 1 shadowless}\n"
"// light_source {<-100, 0, 0> color rgb <1,1,1>  adaptive 1 shadowless}\n"

"background { color White*0.3 }\n"

"#if (PERSPECTIVE)\n"
"  camera { perspective right -x*ASPECTRATIO up y location <0,0,DISTANCE> look_at <0,0,0> }\n"
"#else\n"
"  camera { orthographic right -x*DISTANCE*ASPECTRATIO up y*DISTANCE location <0,0,DISTANCE> look_at <0,0,0> }\n"
"#end\n"

"// Bond Radius / Color\n"
"#declare bondw = 0.1;\n"
	  "#declare bondc = pigment {color rgb <1.0,1.0,0.0>};\n",
	  avg[0],avg[1],avg[2]
	  );
  return 1;
}

static int count_and_label_visible_atom(void)
{
  int i,nn_atom=0;
  for(i=0;i<n_satom;i++){
    SAT[i].temp_num=0;
    if ((SAT[i].view_flag & FLAG_OMIT)==0) {
      nn_atom++;
      SAT[i].temp_num=nn_atom;
    }	
  }
  return nn_atom;
}

SV_Function void povrayout(void) 
{
  FILE *fp;
  int nn_atom,i;
  S_AT *satp;
  Double RMAT[3][3];
  static Double RGBT[][3]={
    /* For Normal */
    {0.1,0.1,0.1},
    {0.1,0.1,1.0},
    {1.0,0.1,0.1},
    {1.0,0.1,1.0},
    {0.1,0.7,0.1},
    {0.1,1.0,1.0},
    {1.0,1.0,0.1},
    {1.0,1.0,1.0},
  };
  fp=ffopen(InputFileName("Export File (Povray)","sv.pov",0),"w");
  if (fp==NULL) cmderr("Can't open sv.pov for temporary");
  nn_atom=count_and_label_visible_atom();
  if (nn_atom==0) {
    cmderr("No visible atoms !");
    return;
  }
  
  fprintf(fp,"// This file was made automatically by 'SV'\n");
  povray_out_header(fp);

  fprintf(fp,"#declare fn=finish { ambient 0.3 metallic reflection 0.2 specular 1 }\n");
  for(i=0;i<n_sfac;i++) {
    fprintf(fp,"#declare cl%s=pigment { color rgb < %.2f %.2f %.2f > };\n",
	    sfac[i].name,
	    RGBT[sfac[i].col][0],
	    RGBT[sfac[i].col][1],
	    RGBT[sfac[i].col][2]
	    );
    fprintf(fp,"#declare fn%s=fn;\n");
    fprintf(fp,"#declare vw%s=%.3f;\n",sfac[i].name,sfac[i].vdw);
  }

  /**** Fractional Coordinate **********/
  inv_matrix(MAT,RMAT);

  fprintf(fp,"#declare ATOMS= union {\n");
  for(i=0;i<n_satom;i++){
    if ((SAT[i].view_flag & FLAG_OMIT)==0) {
      int color;
      satp=SAT+i;
      color=satp->color & 7;
      fprintf(fp,
        " sphere { < %8.3f, %8.3f, %8.3f >, vw%s texture { cl%s fn%s } } // %s\n",
	      satp->vloc[0],satp->vloc[1],satp->vloc[2], 
	      sfac[orgAtom[satp->atm].atom_type-1].name,
	      sfac[orgAtom[satp->atm].atom_type-1].name,
	      sfac[orgAtom[satp->atm].atom_type-1].name,
	      orgAtom[satp->atm].name
	      );
    }	
  }
  fprintf(fp,"};\n");
  fprintf(fp,"#declare BONDS = union {\n");
  for(i=0;i<n_bond;i++){
    if ((SAT[BOND[i].atm[0]].view_flag & FLAG_OMIT)==0 &&
	(SAT[BOND[i].atm[1]].view_flag & FLAG_OMIT)==0 ) {
      fprintf(fp,"cylinder { <%.3f,%.3f,%.3f>,<%.3f,%.3f,%.3f>,bondw texture{bondc}}\n",
	      SAT[BOND[i].atm[0]].vloc[0],
	      SAT[BOND[i].atm[0]].vloc[1],
	      SAT[BOND[i].atm[0]].vloc[2],
	      SAT[BOND[i].atm[1]].vloc[0],
	      SAT[BOND[i].atm[1]].vloc[1],
	      SAT[BOND[i].atm[1]].vloc[2]);
    }
  }
  fprintf(fp,"};\n");

  fprintf(fp,
	  "#declare FULL= union {\n"	 
          "  object {ATOMS}\n"
	  "  object {BONDS}\n"
	  "  translate -CENTER\n"
	  "  rotate <0,clock*360,0>\n"
	  "  translate TRANSLATE\n"
	  "  scale SCALE\n"
	  "};\n"
	  "#if (STEREO)\n"
	  "  #if (PERSPECTIVE)\n"
	  "    #declare srot=atan2(STEREOSPLIT,DISTANCE)/3.1415*180;\n" 
	  "  #else\n"
	  "    #declare srot=0;\n"
	  "  #end\n"
	  "  object {FULL rotate <0,srot+5,0> translate <-STEREOSPLIT,0,0> }\n"
	  "  object {FULL rotate <0,-srot-5,0> translate <STEREOSPLIT,0,0> }\n"
	  "#else\n"
	  "  object {FULL}\n"
	  "#end\n"
	  );

  fclose(fp);
#ifdef POVRAY_EXEC
  system("povray sv.pov");
#endif
}
